#
# APS-18-ID.py  - Site-specific script for Beamline 18-ID at the APS.
#

print "***** Start of APS-18-ID.py *****"

import MpCa
import threading

#---------------------------------------------------------------------------
#
# Create a simple event type used by the EPICS subscription thread to send
# messages to the main thread.  Inspired by
#
#    http://wiki.wxpython.org/LongRunningTasks
#

global EVT_RESULT_ID
global EVT_RESULT
global ResultEvent

EVT_RESULT_ID = wx.NewId()

def EVT_RESULT( win, func ):
	global EVT_RESULT_ID

	win.Connect( -1, -1, EVT_RESULT_ID, func )

class ResultEvent( wx.PyEvent ):
	global EVT_RESULT_ID

	def __init__( self, data ):
		wx.PyEvent.__init__( self )
		self.SetEventType( EVT_RESULT_ID )
		self.data = data

#---------------------------------------------------------------------------
#

global PVOUT_SET_VALUE
global PVOUT_CHECK_IDLE

PVOUT_SET_VALUE = 1
PVOUT_CHECK_IDLE = 2

#---------------------------------------------------------------------------
#

def biocat_setup_pv_display( panel, pv_name ):
	global MpCa

	try:
		del panel.value_callback
	except:
		pass

	try:
		del panel.trigger_pv
	except:
		pass

	try:
		old_panel_value = panel.value
	except:
		old_panel_value = None

	try:
		panel.trigger_pv = MpCa.PV( pv_name )

		pv_value = panel.trigger_pv.caget()

		try:
			panel.value.SetLabel( str(pv_value) )
			panel.value.SetForegroundColour( "blue" )
		except:
			#print "PV %s label not set, due to no panel.value" \
			#	% pv_name
			pass
	except:
		panel.trigger_pv = None

		try:
			panel.value.SetLabel( "PV %s NOT FOUND" % pv_name )
			panel.value.Enable(False)
		except:
			#print "PV NOT FOUND not set, due to no panel.value"
			pass

	if ( panel.trigger_pv != None ):
		panel.value_callback = panel.trigger_pv.add_callback(
			MpCa.DBE_VALUE, panel.trigger_callback_handler, panel )
	else:
		panel.value_callback = None

	return

#---------------------------------------------------------------------------
#

def biocat_ad_trigger():
	print "biocat_ad_trigger() invoked."

	current_sequence_panel = \
	_mpad.frame.image_tab.right_panel.sequence_panel.current_sequence_panel

	print "current_sequence_panel =", current_sequence_panel

	sequence_panel_command = current_sequence_panel.command

	print "sequence_panel_command =", sequence_panel_command

	#------

	detector_controls_tab = _mpad.frame.detector_controls_tab
	input_trigger_panel = \
			detector_controls_tab.pv_trigger_panel.input_panel
	output_trigger_panel = \
			detector_controls_tab.pv_trigger_panel.output_panel

	trigger_pv = output_trigger_panel.trigger_pv

	#------

	input_enabled  = input_trigger_panel.checkbox.GetValue()
	output_enabled = output_trigger_panel.checkbox.GetValue()

	#------

	if ( input_enabled ):
		wx.Yield()

	#------

	if ( output_enabled == False ):
		print "biocat_ad_trigger(): trigger delay disabled."
	else:
		trigger_delay = \
			output_trigger_panel.trigger_delay_ctrl.GetValue()

		trigger_delay = float(trigger_delay)

		#------

		trigger_value = \
			output_trigger_panel.trigger_value_ctrl.GetValue()

		trigger_value = int(trigger_value)

		#------

		if ( trigger_pv == None ):
			print \
	"WARNING: Skipping output PV trigger since the PV is not connected!"
		else:
			print \
		"biocat_ad_trigger(): Sending the output trigger value %d" \
							% trigger_value

			MpCa.poll()
			trigger_pv.caput( trigger_value )
			MpCa.poll()

		#------

		print "biocat_ad_trigger(): Waiting for %f seconds..." \
						% trigger_delay

		time.sleep( trigger_delay )

		print "biocat_ad_trigger(): Finished wait."

	print "biocat_ad_trigger(): Triggering the area detector!"

	sequence_panel_command( current_sequence_panel )

	if ( output_enabled ):
		wx.Yield()

		if ( trigger_pv != None ):
			#
			# Post an event to the output trigger panel telling
			# it to check to see if the area detector is idle.
			# If it is idle, the output trigger will be reset.
			#

			wx.PostEvent( output_trigger_panel, \
					ResultEvent( PVOUT_CHECK_IDLE ) )

	print "biocat_ad_trigger() complete."

#---------------------------------------------------------------------------

class MpadBiocatInputTriggerPanel( wx.Panel ):
	global pv_input_value_callback
	global pv_input_trigger_name_update
	global biocat_ad_trigger
	global biocat_setup_pv_display
	global EVT_RESULT

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.epics_initialized = False

		self.trigger_pv = None
		self.trigger_callback = None

		self.grid = wx.GridBagSizer( hgap=5, vgap=5 )
		self.SetSizer( self.grid )

		row = 0

		#------

		self.entry_label = wx.StaticText( self, -1, "PV Name:" )

		self.grid.Add( self.entry_label, pos=(row,0), \
			flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		self.trigger_row = row + 1
		self.trigger_callback_handler = pv_input_value_callback

		pv_name_record = "pvin_trig_name.value"

		self.pv_name = MpWx.ValueEntry( self,
				_mpad.server_record, pv_name_record,
				size=(200,-1),
				function=pv_input_trigger_name_update )

		self.grid.Add( self.pv_name, pos=(row,1) )

		#------

		self.checkbox = wx.CheckBox( self, -1, "Enable" )

		self.grid.Add( self.checkbox, pos=(row,2) )

		row = row + 1

		#------

		self.value_label = wx.StaticText( self, -1, "PV Value:" )

		self.grid.Add( self.value_label, pos=(row,0) )

		#------

		self.value = wx.StaticText( self, -1, "--------" )

		self.grid.Add( self.value, pos=(row,1) )

		trigger_pv_name = self.pv_name.GetValue()

		biocat_setup_pv_display( self, trigger_pv_name )

		row = row + 1

		#------

		self.trigger_value_label = wx.StaticText( self, -1, \
							"PV Trigger Value:" )

		self.grid.Add( self.trigger_value_label, pos=(row,0), \
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_value_record = "pvin_trig_val.value"

		self.trigger_value_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_value_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_value_ctrl, pos=(row,1),
				flag = wx.ALIGN_CENTER_VERTICAL )
		
		#------

		EVT_RESULT( self, self.OnResult )

		#------

	def OnResult( self, event ):

		new_value = self.trigger_pv.get_local()

		new_value = int(new_value)

		print "PV Input OnResult(): new_value =", new_value

		trigger_value = self.trigger_value_ctrl.GetValue()

		trigger_value = int(trigger_value)

		print "PV Input OnResult(): trigger_value =", trigger_value

		self.value.SetLabel( str(new_value) )

		self.value.Enable(True)

		cb_state = self.checkbox.GetValue()

		if ( cb_state == False ):
			print "PV Input OnResult(): Trigger not enabled."
			return

		if ( new_value != trigger_value ):
			print \
	"PV Input OnResult(): New value %d did not match trigger value %d." \
				% ( new_value, trigger_value )
			return

		print "PV Input OnResult(): Triggering the detector."

		biocat_ad_trigger()

		print "PV Input OnResult(): Resetting the input trigger."

		MpCa.poll()

		self.trigger_pv.caput( 0, wait=False )

		MpCa.poll()

		print "PV Input OnResult() complete."

#------
#
# Note: pv_input_value_callback() is invoked in the context of the
#       EPICS subscription callback thread.  However, we can only
#       perform GUI operations from the main thread, so we must
#       post an event to the main thread to tell the input trigger
#       panel to invoke its OnResult() method.
#

def pv_input_value_callback( callback, args ):

	trigger_panel = args

	wx.PostEvent( trigger_panel, ResultEvent(None) )

#------

def pv_input_trigger_name_update( nf, widget, args, value ):

	print "pv_input_trigger_name_update(): value =", value

	trigger_panel = widget.parent

	biocat_setup_pv_display( trigger_panel, value )

	MpWx._ValueEntry_update( nf, widget, args, value )

	# print "pv_input_trigger_name_update() complete."

#---------------------------------------------------------------------------
#

class MpadBiocatOutputTriggerPanel( wx.Panel ):
	global pv_output_value_callback
	global pv_output_trigger_name_update
	global EVT_RESULT

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=5, vgap=5 )
		self.SetSizer( self.grid )

		row = 0

		#------

		self.entry_label = wx.StaticText( self, -1, "PV Name:" )

		self.grid.Add( self.entry_label, pos=(row,0),
			flag = wx.ALIGN_CENTER_VERTICAL )
		
		#------

		self.trigger_row = row + 1
		self.trigger_callback_handler = pv_output_value_callback

		pv_name_record = "pvout_trig_name.value"

		self.pv_name = MpWx.ValueEntry( self,
				_mpad.server_record, pv_name_record,
				size=(200,-1),
				function=pv_output_trigger_name_update )

		self.grid.Add( self.pv_name, pos=(row,1),
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		#------

		self.checkbox = wx.CheckBox( self, -1, "Enable" )

		self.grid.Add( self.checkbox, pos=(row,2) )

		row = row + 1

		#------

		self.value_label = wx.StaticText( self, -1, "PV Value" )

		self.grid.Add( self.value_label, pos=(row,0) )

		#------

		self.value = wx.StaticText( self, -1, "--------" )

		self.grid.Add( self.value, pos=(row,1) )

		trigger_pv_name = self.pv_name.GetValue()

		biocat_setup_pv_display( self, trigger_pv_name )

		row = row + 1

		#------

		self.trigger_value_label = wx.StaticText( self, -1,
							"PV Trigger Value:" )

		self.grid.Add( self.trigger_value_label, pos=(row,0),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_value_record = "pvout_trig_val.value"

		self.trigger_value_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_value_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_value_ctrl, pos=(row,1),
				flag=wx.ALIGN_CENTER_VERTICAL )

		row = row + 1

		#------

		self.trigger_delay_label = wx.StaticText( self, -1,
							"Trigger Delay:" )

		self.grid.Add( self.trigger_delay_label, pos=(row,0),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_delay_record = "pvout_delay.value"

		self.trigger_delay_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_delay_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_delay_ctrl, pos=(row,1),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		self.trigger_delay_units = wx.StaticText( self, -1, "seconds" )

		self.grid.Add( self.trigger_delay_units, pos=(row,2),
				flag=wx.ALIGN_CENTER_VERTICAL )

		EVT_RESULT( self, self.OnResult )

	def OnResult( self, event ):

		global threading
		global pv_output_timer_callback

		if ( event.data == PVOUT_SET_VALUE ):
			new_value = self.trigger_pv.get_local()

			print "PVOUT_SET_VALUE: new_value =", new_value

			self.value.SetLabel( str(new_value) )

			self.value.Enable(True)

		elif ( event.data == PVOUT_CHECK_IDLE ):
			ad_status = _mpad.ad_record.get_status()

			busy = ad_status & Mp.MXSF_AD_IS_BUSY

			print "PVOUT_CHECK_IDLE: busy =", busy

			# If the area detector is finished, then
			# reset the output trigger PV.

			if ( busy == 0 ):
				print \
				"PVOUT_CHECK_IDLE: Resetting output trigger PV."

				MpCa.poll()
				self.trigger_pv.caput( 0 )
				MpCa.poll()

				return

			# Otherwise, we must reschedule a check of the
			# area detector status for later.

			delay = 1.0		# In seconds

			t = threading.Timer( delay, pv_output_timer_callback, \
							args = [ self ] )

			t.start()

			print "PVOUT_CHECK_IDLE rescheduled."

#------
#
# Note: pv_output_value_callback() is invoked in the context of the
#       EPICS subscription callback thread.  However, we can only
#       perform GUI operations from the main thread, so we must
#       post an event to the main thread to tell the output trigger
#       panel to invoke its OnResult() method.
#

def pv_output_value_callback( callback, args ):

	trigger_panel = args

	wx.PostEvent( trigger_panel, ResultEvent( PVOUT_SET_VALUE ) )

#------

def pv_output_trigger_name_update( nf, widget, args, value ):

	print "pv_output_trigger_name_update(): value =", value

	trigger_panel = widget.parent

	biocat_setup_pv_display( trigger_panel, value )

	MpWx._ValueEntry_update( nf, widget, args, value )

	# print "pv_output_trigger_name_update() complete."

#------
#

def pv_output_timer_callback( output_trigger_panel ):
	#
	# Post an event to the output trigger panel telling
	# it to check to see if the area detector is idle.
	# If it is idle, the output trigger will be reset.
	#

	wx.PostEvent( output_trigger_panel, ResultEvent( PVOUT_CHECK_IDLE ) )

#---------------------------------------------------------------------------
#

class MpadBiocatPVTriggerPanel( wx.Panel ):
	global MpadBiocatInputTriggerPanel
	global MpadBiocatOutputTriggerPanel

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.last_poll_time = time.time()

		#----

		self.input_box = wx.StaticBox( self, -1, "PV Input Trigger" )

		self.input_box_sizer = wx.StaticBoxSizer( self.input_box )

		self.input_panel = MpadBiocatInputTriggerPanel( self )

		self.input_box_sizer.Add( self.input_panel, 1, wx.GROW )

		MpCa.poll()
		print "Sleeping ..."
		time.sleep(2)

		#----

		self.output_box = wx.StaticBox( self, -1, "PV Output Trigger" )

		self.output_box_sizer = wx.StaticBoxSizer( self.output_box )

		self.output_panel = MpadBiocatOutputTriggerPanel( self )

		self.output_box_sizer.Add( self.output_panel, 1, wx.GROW )

		MpCa.poll()

		#----

		self.sizer = wx.BoxSizer( wx.HORIZONTAL )

		self.sizer.Add( self.input_box_sizer )
		self.sizer.Add( self.output_box_sizer )

		self.SetSizerAndFit( self.sizer )

		self.Layout()

		self.Bind( wx.EVT_IDLE, self.OnIdle )

	def OnIdle( self, event ):
		# Allow EPICS events to be processed.

		current_time = time.time()

		if ( (current_time - self.last_poll_time) > 5.0 ):
			MpCa.poll()

			self.last_poll_time = current_time


#-----------------------------------------------------------------------------

class MpadBiocatShutterControlPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=5, vgap=5 )

		self.SetSizer( self.grid )

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		#
		# FIXME - We should get this shutter dictionary from
		# some beamline specific file.
		#

		self.shutter_dict = { 'Script Shutter':'script_shutter', \
				'X-Ray Shutter':'xray_shutter' }

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		self.label = wx.StaticText( self, -1, "Shutter: " )

		self.grid.Add( self.label, pos=(0,0), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		self.shutter_selector = wx.Choice( self, -1, \
					choices=self.shutter_dict.keys() );

		self.shutter_selector.SetSelection(0)

		self.Bind( wx.EVT_CHOICE, self.OnSelect, self.shutter_selector )

		self.grid.Add( self.shutter_selector, pos=(0,1), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		#-----

		# Start out by selecting the first record in the dictionary.

		selection = self.shutter_dict.keys()[0]

		self.register_selection( selection )

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		self.shutter_button = wx.Button( self, \
			label="________________" )

		self.grid.Add( self.shutter_button, pos=(1,1), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		self.Bind( wx.EVT_BUTTON, self.OnShutterButton, \
					self.shutter_button )

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		# FIXME - It doesn't look like a real LED.

		self.shutter_led = wx.lib.stattext.GenStaticText( self, \
				-1, "", size=(20,20) )

		self.shutter_led.SetBackgroundColour( "white" )

		self.grid.Add( self.shutter_led, pos=(1,2), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		wx.FutureCall( 500, self.shutter_monitor );

	#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	def OnShutterButton( self, event ):

		if ( self.shutter_status == 0 ):
			_mpad.shutter_record.relay_command( 1 )
		elif ( self.shutter_status == 1 ):
			_mpad.shutter_record.relay_command( 0 )
		else:
			print "Nothing happened."

	def shutter_monitor( self ):
		try:
			# Get the status of the currently selected shutter.

			self.shutter_status = \
				_mpad.shutter_record.get_relay_status()

			if ( self.shutter_status == 0 ):
				self.shutter_led.SetBackgroundColour("red")
				self.shutter_button.SetLabel("Close Shutter")
			elif ( self.shutter_status == 1 ):
				self.shutter_led.SetBackgroundColour("black")
				self.shutter_button.SetLabel("Open Shutter")
			else:
				self.shutter_led.SetBackgroundColour("yellow")
				self.shutter_button.SetLabel("Xyzzy")

			self.shutter_button.Enable(True)
		except:
			pass

		wx.FutureCall( 500, self.shutter_monitor );

	#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	def OnSelect( self, event ):
		selection = event.GetString()

		# Disable the button until the next call to shutter_monitor().

		self.shutter_button.Enable(False)

		# Look for the selected shutter record.

		self.register_selection( selection )

	def register_selection( self, selection ):
		record_name = self.shutter_dict[selection]

		try:
			_mpad.shutter_record = \
				_mpad.record_list.get_record(record_name)

		except Mp.Not_Found_Error:
			_mpad.shutter_record = None

			print "Error: The shutter record '%s' was not found." \
				% ( record_name )


#---------------------------------------------------------------------------
#
# Find the Detector Controls tab and insert MpadBiocatPVTriggerPanel
# before the MpadUserRegisterPanel.
#

detector_controls_tab = _mpad.frame.detector_controls_tab

user_register_panel_index = None

detector_controls_children = detector_controls_tab.box.GetChildren()

for child in detector_controls_children:

	child_window = child.GetWindow()

	if isinstance( child_window, MpadUserRegisterPanel ):
		user_register_panel_index = \
			detector_controls_children.index( child )
		break

if ( user_register_panel_index == None ):
	print "MpadUserRegisterPanel not found!  Aborting!"
	sys.exit(0)

#detector_controls_tab.pv_trigger_panel = \
#			MpadBiocatPVTriggerPanel( detector_controls_tab )

#detector_controls_tab.box.Insert( user_register_panel_index,
#				detector_controls_tab.pv_trigger_panel,
#				border=10, flag=(wx.TOP | wx.ALIGN_CENTER) )

#------

def Biocat_OnStartButton( event ):
	print "Biocat_OnStartButton() invoked."

	biocat_ad_trigger()

	print "Biocat_OnStartButton() complete."

#---------------------------------------------------------------------------
#
# Look in the Right Panel in the Image tab for the Start Sequence button.
#

mpad_right_panel = _mpad.frame.image_tab.right_panel

#
# Replace the existing handler for the Start Sequence button.
#

mpad_right_panel.Bind( wx.EVT_BUTTON,
		Biocat_OnStartButton, mpad_right_panel.start_button )

#

#---------------------------------------------------------------------------
#
# Insert MpadBiocatShutterControlPanel after the Stop Sequence button.
#

#
# First we must find the Stop Sequence button in the list of items
# managed by the panel's sizer.
#

stop_sequence_index = None

right_panel_children = mpad_right_panel.box.GetChildren()

for child in right_panel_children:

	child_window = child.GetWindow()

	if isinstance( child_window, wx.Button ):
		button_label = child_window.GetLabel()

		if ( button_label == "Stop Sequence" ):
			stop_sequence_index = \
				right_panel_children.index( child )
			break

if ( stop_sequence_index == None ):
	print "Stop Sequence button not found in right panel!  Aborting!"
	sys.exit(0)

#
# We want to insert our new widgets after the Stop Sequence button.
# However, WxPython does not give us a way of directly doing that.
# What it _does_ give us is a way of inserting before an item.
# So we must find the _next_ item and insert before that.
#

next_item_index = stop_sequence_index + 1

#
# Insert two separators.
#

mpad_right_panel.biocat_shutter_sep1 = wx.StaticLine( mpad_right_panel )

mpad_right_panel.box.Insert( next_item_index,
		mpad_right_panel.biocat_shutter_sep1, border=10,
		flag=(wx.TOP | wx.ALIGN_LEFT | wx.EXPAND) )

next_item_index += 1

mpad_right_panel.biocat_shutter_sep2 = wx.StaticLine( mpad_right_panel )

mpad_right_panel.box.Insert( next_item_index,
		mpad_right_panel.biocat_shutter_sep2, border=5,
		flag=(wx.BOTTOM | wx.ALIGN_LEFT | wx.EXPAND) )

next_item_index += 1

#
# Create the shutter control panel.
#

mpad_right_panel.biocat_shutter_control = \
		MpadBiocatShutterControlPanel( mpad_right_panel )

mpad_right_panel.box.Insert( next_item_index,
		mpad_right_panel.biocat_shutter_control, border=5,
		flag=(wx.TOP | wx.ALIGN_CENTER) )

if 1:
	# Force a redraw of the entire GUI.

	_mpad.frame.image_tab.Layout()
	_mpad.frame.Layout()

	_mpad.frame.Fit()

	_mpad.frame.Refresh()
	_mpad.frame.Update()

#------

print "***** End of APS-18-ID.py *****"

