#
#----------------------------------------------------------------------------
#

class MpadDetectorControlsTab(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		#
		# Create a Box sizer to contain the controls.
		#

		self.box = wx.BoxSizer( wx.VERTICAL )

		#
		# Attach the sizer to the panel.
		#

		self.SetSizer( self.box )

		#
		# Display the detector name at the top of the panel.
		#

		self.name = wx.StaticText( self, \
				-1, "Aviex PCCD-9785 Controls" )

		self.box.Add( self.name, flag=wx.ALIGN_CENTER )

		#
		# Display control register values.
		#

		self.control_register_panel = MpadControlRegisterPanel( self )

		self.box.Add( self.control_register_panel, flag=wx.ALIGN_CENTER)

		#
		# Display register settings
		#

		self.user_register_panel = MpadUserRegisterPanel( self )

		self.box.Add( self.user_register_panel, flag=wx.ALIGN_CENTER )

#----------------------------------------------------------------------------

class MpadDetectorReadoutBox(MpWx.RadioBox):

	def OnRadioBox( self, event ):

		selection = self.GetSelection()

		if ( selection == 0 ):
			value = 0
		elif ( selection == 1 ):
			value = 2
		elif ( selection == 2 ):
			value = 3
		else:
			print "Illegal detector readout box selection =", \
							selection

		print "MpadDetectorReadoutBox selection = %d, value = %d" \
			% ( selection, value )

		self.nf.put( value )

#----------------------------------------------------------------------------

class MpadControlRegisterPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		detector_readout_choices = \
			[ 'Full frame', 'Sub-image', 'Streak Camera' ]

#		self.detector_readout_box = MpWx.RadioBox( self, \

		self.detector_readout_box = MpadDetectorReadoutBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_detector_readout_mode", \
			label="Detector Readout Mode", pos=(10,10), \
			choices = detector_readout_choices, \
			style=wx.RA_SPECIFY_ROWS )

		readout_speed_choices = [ 'Low-noise', 'High-speed' ]

		self.readout_speed_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_readout_speed", \
			label="Readout Speed", pos=(170, 10), \
			choices = readout_speed_choices, \
			style=wx.RA_SPECIFY_ROWS )

		test_mode_choices = [ 'Off', 'On' ]

		self.test_mode_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_test_mode", \
			label="Test Mode", pos=(280, 10), \
			choices = test_mode_choices, \
			style=wx.RA_SPECIFY_ROWS )

		offset_correction_choices = [ 'Off', 'On' ]

		self.automatic_offset_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_offset_correction", \
			label="Automatic Offset Correction", pos=(365, 10), \
			choices = test_mode_choices, \
			style=wx.RA_SPECIFY_ROWS )

		exposure_mode_choices = [ 'Internal (software)', 'External' ]

		self.exposure_mode_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_exposure_mode", \
			label="Exposure Trigger", pos=(550, 10), \
			choices = exposure_mode_choices, \
			style=wx.RA_SPECIFY_ROWS )

#		external_trigger_choices = [ 'Edge Triggered', 'Duration' ]
#
#		self.external_trigger_box = MpWx.RadioBox( self, \
#			_mpad.server_record, \
#			_mpad.remote_ad_name + ".dh_external_trigger", \
#			label="External Trigger Type", pos=(698, 10), \
#			choices = external_trigger_choices, \
#			style=wx.RA_SPECIFY_ROWS )

		_mpad.control_register_nf = mpad_monitor_display(
					"dh_control", self, 1, 0, \
					control_update, None )

	def OnDetectorReadoutBox( self, event ):
		selection = self.detector_readout_box.GetSelection()

		if ( selection == 0 ):
			value = 0
		elif ( selection == 1 ):
			value = 2
		elif ( selection == 2 ):
			value = 3
		else:
			print "Illegal detector_readout_box selection =", \
							selection

		mpad_set_control_register( value, 3, 2 )

#---

def mpad_set_control_register( value, offset, width ):

	mask = 0

	for i in range(width):
		mask <<= 1
		mask |= 1

	mask <<= offset

	new_value = value << offset

	new_value &= mask

	control_register = _mpad.control_register_nf.get()

	new_control_register = control_register & ~mask

	new_control_register |= new_value

	if ( _mpad.debug_serial ):
		print "**********************"
		print "Setting control register to %#x" % new_control_register
		print "**********************"

	_mpad.control_register_nf.put( new_control_register )

#---

def control_update( nf, not_used1, not_used2 ):

	control_panel = _mpad.frame.detector_controls_tab.control_register_panel

	setup_panel = _mpad.frame.setup_parameters_tab.setup_controls_panel

	control_register = nf.get()

	if ( _mpad.debug_serial ):
		print "Received %#x for '%s'" % ( control_register, nf.name )

	#---

	detector_readout_mode = ( control_register >> 3 ) & 0x3

	if ( detector_readout_mode == 0 ):
		control_panel.detector_readout_box.SetSelection(0)

	elif ( detector_readout_mode == 2 ):
		control_panel.detector_readout_box.SetSelection(1)

	elif ( detector_readout_mode == 3 ):
		control_panel.detector_readout_box.SetSelection(2)

	#---

	readout_speed = ( control_register >> 8 ) & 0x1

	control_panel.readout_speed_box.SetSelection( readout_speed )

	#---

	test_mode = ( control_register >> 9 ) & 0x1

	control_panel.test_mode_box.SetSelection( test_mode )

	#---

	automatic_offset_correction = ( control_register >> 7 ) & 0x1

	control_panel.automatic_offset_box.SetSelection( \
					automatic_offset_correction )

	#---

	exposure_mode = ( control_register >> 6 ) & 0x1

	control_panel.exposure_mode_box.SetSelection( exposure_mode )

	#---

	external_trigger = ( control_register >> 5 ) & 0x1

	control_panel.external_trigger_box.SetSelection( external_trigger )

	#---

	linearization = ( control_register >> 2 ) & 0x1

	setup_panel.linearization_box.SetSelection( linearization )

	#---

	offset_correction = ( control_register >> 1 ) & 0x1

	setup_panel.offset_correction_box.SetSelection( offset_correction )

#----------------------------------------------------------------------------

class MpadUserRegisterPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=10 )

		self.SetSizer( self.grid )

		#----

		allowed_bin_sizes = [ 1, 2, 4, 8, 16, 32, 64, 128 ]

		self.hbin_ctrl = MpadBinningControl( self, \
					"dh_pixel_binning", \
					self.grid, 0, \
					"Pixel Binning (H):", \
					allowed_bin_sizes )

		self.vbin_ctrl = MpadBinningControl( self, \
					"dh_line_binning", \
					self.grid, 1, \
					"Line Binning (V):", \
					allowed_bin_sizes )

		#----

		self.exposure_time_ctrl = MpadRegisterControl( self, \
					"dh_exposure_time", \
					self.grid, 2, \
					"Exposure time:", \
					"      seconds", \
					0.0, 0.01, 65535, 0.01, 0 )

		self.num_frame_in_sequence_ctrl = MpadRegisterControl( self, \
					"dh_frames_per_sequence", \
					self.grid, 5, \
					"Number of frames in sequence:", \
					"", \
					1, 1, 127, 1, 0 )

		self.gap_time_ctrl = MpadRegisterControl( self, \
					"dh_gap_time", \
					self.grid, 6, \
					"Gap time:", \
					"      seconds", \
					0.0, 0.0001, 65535, 0.0001, 0 )

		#----

		allowed_subframe_size = [ 16, 32, 64, 128, 256, 512, 1024 ]

		self.num_subframe_size_ctrl = MpadBinningControl( self, \
					"dh_subframe_size", \
					self.grid, 9, \
					"Subframe size in lines:", \
					allowed_subframe_size )

		#----

		self.num_subimages_per_read_ctrl = MpadRegisterControl( self, \
					"dh_subimages_per_read", \
					self.grid, 10, \
					"Number of sub-images per read:", \
					"", \
					1, 1, 127, 1, 0 )

		self.num_streak_camera_lines_ctrl = MpadRegisterControl( self, \
					"dh_streak_mode_lines", \
					self.grid, 11, \
					"Number of streak camera lines:", \
					"", \
					1, 1, 65534, 1, 0 )

#----------------------------------------------------------------------------

class MpadBinningControl:

	def __init__( self, parent, field_name, grid, row, label, \
						allowed_bin_sizes ):

		field_name = _mpad.remote_ad_name + "." + field_name

		self.parent = parent

		choice_list = []

		for i in allowed_bin_sizes:
			choice_list.append( str(i) )

		self.label = wx.StaticText( parent, -1, label )

		grid.Add( self.label, pos=(row,0), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		self.choice = MpWx.Choice( parent, \
					_mpad.server_record, field_name, \
					choices=choice_list )

		grid.Add( self.choice, pos=(row,1) )

#----------------------------------------------------------------------------

class MpadRegisterControl:

	def __init__( self, parent, field_name, grid, row, label, units, \
			start, step_size, num_steps, scale, offset ):

		field_name = _mpad.remote_ad_name + "." + field_name

		self.parent = parent

		self.start = start
		self.step_size = step_size
		self.num_steps = num_steps

		nf = Mp.Net( _mpad.server_record, field_name )

		read_only = nf.get_attribute( Mp.MXNA_READ_ONLY )

		self.label = wx.StaticText( parent, -1, label )

		grid.Add( self.label, pos=(row,0), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		if ( read_only ):
			self.value = MpWx.Value( parent, \
					_mpad.server_record, field_name )

			self.value.SetForegroundColour( "blue" )
		else:
			self.value = MpWx.ValueEntry( parent, \
					_mpad.server_record, field_name )

		grid.Add( self.value, pos=(row,1), \
			flag=(wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL) )

		self.units = wx.StaticText( parent, -1, units )

		grid.Add( self.units, pos=(row,2), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

#
#----------------------------------------------------------------------------
#

class MpadSetupParametersTab(wx.Panel):

	def __init__( self, parent ):

		wx.Panel.__init__( self, parent )

		self.parent = parent

		#
		# Create a sizer to contain the controls.
		#

		self.box = wx.BoxSizer(wx.VERTICAL)

		#
		# Attach the sizer to the panel.
		#

		self.SetSizer( self.box )

		#
		# Display the detector name at the top of the panel.
		#

		self.name = wx.StaticText( self, \
				-1, "Aviex PCCD-9785 Setup Parameters" )

		self.box.Add( self.name, flag=wx.ALIGN_CENTER )

		#----

		#
		# Display control register values.
		#

		self.setup_controls_panel = MpadSetupControlsPanel( self )

		self.box.Add( self.setup_controls_panel, flag=wx.ALIGN_CENTER)

		#
		# Display register settings
		#

		self.setup_register_panel = MpadSetupRegisterPanel( self )

		self.box.Add( self.setup_register_panel, flag=wx.ALIGN_CENTER )

#----------------------------------------------------------------------------

class MpadSetupControlsPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		linearization_choices = [ 'Off', 'On' ]

		self.linearization_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_linearization", \
			label="Linearization", pos=(10,10), \
			choices = linearization_choices, \
			style=wx.RA_SPECIFY_ROWS )

		offset_correction_choices = \
		    [ 'Binned Pixel Averaging', 'Un-binned Pixel Averaging' ]

		self.offset_correction_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_offset_correction", \
			label="Offset Correction Mode", pos=(100,10), \
			choices = offset_correction_choices, \
			style=wx.RA_SPECIFY_ROWS )

#----------------------------------------------------------------------------

class MpadSetupRegisterPanel(wx.Panel):

	def __init__( self, parent ):

		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=10 )

		self.SetSizer( self.grid )

		self.physical_lines_in_quadrant_ctrl = \
				MpadRegisterControl( self, \
				"dh_physical_lines_in_quadrant", \
				self.grid, 1, \
				"Number of physical lines in quadrant:", \
				"", \
				1, 1, 8191, 1, 0 )

		self.physical_pixels_per_line_in_quadrant_ctrl = \
				MpadRegisterControl( self, \
				"dh_physical_pixels_per_line_in_quadrant", \
				self.grid, 2, \
			"Number of physical pixels per line in quadrant:", \
				"", \
				1, 1, 8191, 1, 0 )

		self.lines_read_in_quadrant_ctrl = \
				MpadRegisterControl( self, \
				"dh_lines_read_in_quadrant", \
				self.grid, 3, \
				"Number of lines read in quadrant:", \
				"", \
				1, 1, 8191, 1, 0 )

		self.pixels_read_in_quadrant_ctrl = \
				MpadRegisterControl( self, \
				"dh_pixels_read_in_quadrant", \
				self.grid, 4, \
				"Number of pixels read in quadrant:", \
				"", \
				1, 1, 8191, 1, 0 )

		#----

		self.controller_fpga_version_name = wx.StaticText( self, -1, \
						"Controller FPGA Version:" )

		self.grid.Add( self.controller_fpga_version_name, \
				pos=(21,0), border=5, \
				flag=(wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL))

		self.controller_fpga_version_value = wx.StaticText( self, -1, \
						"-----" )

		self.controller_fpga_version_value.SetForegroundColour( "blue" )

		self.grid.Add( self.controller_fpga_version_value, \
				pos=(21,1), border=5, \
				flag=(wx.ALIGN_CENTER))

		field_name = "%s.dh_controller_fpga_version" \
					% (_mpad.ad_record.name)

		field_value = Mp.Net(_mpad.server_record, field_name).get()

		field_value = str(field_value)

		self.controller_fpga_version_value.SetLabel(field_value)

		#----

#		self.communications_fpga_version_name = \
#				wx.StaticText( self, -1, \
#				"Communications FPGA Version:" )
#
#		self.grid.Add( self.communications_fpga_version_name, \
#				pos=(22,0), border=5, \
#				flag=(wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL))
#
#		self.communications_fpga_version_value = \
#				wx.StaticText( self, -1, "-----" )
#
#		self.communications_fpga_version_value.SetForegroundColour( \
#								"blue" )
#
#		self.grid.Add( self.communications_fpga_version_value, \
#				pos=(22,1), border=5, \
#				flag=(wx.ALIGN_CENTER))
#
#		field_name = "%s.dh_comm_fpga_version" % (_mpad.ad_record.name)
#
#		field_value = Mp.Net(_mpad.server_record, field_name).get()
#
#		field_value = str(field_value)
#
#		self.communications_fpga_version_value.SetLabel(field_value)
#
